/*****************************************************************************\
* (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#include "CaloFutureUtils/CellNeighbour.h"
#include "CaloDet/DeCalorimeter.h"
#include "CaloKernel/CaloException.h"
#include <algorithm>

// ============================================================================
/** @file CellNeighbour.cpp
 *
 *  Implementation file for class : CellNeighbour
 *
 *  @author Vanya Belyaev Ivan.Belyaev@itep.ru
 *  @date 25/11/2001
 */
// ============================================================================

// ============================================================================
/** the major method. It returns "true" if the cell
 *  is a neighbor for seed cell
 *  @exeption CaloException if detector information is not available
 *  @param seed  the seed   cell
 *  @param cell  the cell to be tested
 *  @return fraction of area of the second cell which fits inside
 *          "neighbour" aroud the seed cell, equivalently 0 or 1
 */
// ============================================================================
double CellNeighbour::operator()( LHCb::Detector::Calo::CellID seed, LHCb::Detector::Calo::CellID cell ) const {
  if ( seed == cell ) { return 1; }
  if ( !det() ) { Exception( "CellNeighbour:: Detector points to NULL!" ); }
  if ( !det()->valid( seed ) || !det()->valid( cell ) ) { return 0; }
  const auto& neighbours = det()->neighborCells( seed );
  auto        it         = std::find( neighbours.begin(), neighbours.end(), cell );
  return neighbours.end() == it ? 0.0 : 1.0;
}
