#!/usr/bin/env python
###############################################################################
# (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
# =============================================================================
# =============================================================================
## @file DetDescChecks/MaterialBidget.py
#  Helper file to illustrate the usage of Material-Budget algorithm
#  @see DetDesc::CheckOverlap
#  @author Vanya BELYAEV Icvan.Belyaev@nikhef.nl
# =============================================================================
"""
Helper file to illustrate the usage of Material-Budget algorithm
"""

from __future__ import print_function

# =============================================================================
__author__ = "Vanya BELYAEV Icvan.Belyaev@nikhef.nl"
__version__ = "1.1"
# =============================================================================

from Configurables import DDDBConf
from Gaudi.Configuration import *

DDDBConf(DataType="2015")

from Configurables import DetDesc__MaterialBudget

budget = DetDesc__MaterialBudget(
    "Budget",
    #
    zPlane=970.0,
    Shots=100,
    #
    Grid=False,
    Rapidity=False,
    #
)

from Configurables import ApplicationMgr

ApplicationMgr(TopAlg=[budget], EvtSel="NONE", EvtMax=1)

# =============================================================================
if "__main__" == __name__:
    print(__doc__)
    print(__author__)
    print(__version__)

    from GaudiPython.Bindings import AppMgr

    gaudi = AppMgr()

    gaudi.run(1)

# =============================================================================
# The END
# =============================================================================
