/*****************************************************************************\
* (c) Copyright 2021 CERN for the benefit of the LHCb Collaboration           *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#include "Event/CrossSectionsFSR.h"

namespace LHCb {
  const std::map<CrossSectionsFSR::CrossSectionKey, std::string> CrossSectionsFSR::csk2s = {

      { CrossSectionKey::MBCrossSection, "Total cross-section of Minimum Bias events" },

      { CrossSectionKey::MeanNoZeroPUInt, "Mean number of non-empty generated Pile-Up interactions" },
      { CrossSectionKey::MeanPUInt, "Mean number of generated Pile-Up interactions" },
      { CrossSectionKey::MeanPUIntAcc, "Mean number of accepted Pile-Up interactions" },

      { CrossSectionKey::FullEvtCut, "Full event cut efficiency" },

      { CrossSectionKey::bCrossGen, "Generated b cross-section" },
      { CrossSectionKey::ThreebCrossGen, "Double generated b cross-section" },
      { CrossSectionKey::PromptBCrossGen, "Generated prompt B cross-section" },
      { CrossSectionKey::cCrossGen, "Generated c cross-section" },
      { CrossSectionKey::ThreecCrossGen, "Double generated c cross-section" },
      { CrossSectionKey::PromptCCrossGen, "Generated prompt C cross-section" },
      { CrossSectionKey::bAndcCrossGen, "Generated b and c cross-section" },
      { CrossSectionKey::bCrossAcc, "Accepted b cross-section" },
      { CrossSectionKey::ThreebCrossAcc, "Double accepted b cross-section" },
      { CrossSectionKey::PromptBCrossAcc, "Accepted prompt B cross-section" },
      { CrossSectionKey::cCrossAcc, "Accepted c cross-section" },
      { CrossSectionKey::ThreecCrossAcc, "Double accepted c cross-section" },
      { CrossSectionKey::PromptCCrossAcc, "Accepted prompt C cross-section" },
      { CrossSectionKey::bAndcCrossAcc, "Accepted b and c cross-section" },

      { CrossSectionKey::GenLevelCut, "Generator level cut efficiency" },

      { CrossSectionKey::FractionB0Gen, "Fraction of generated B0" },
      { CrossSectionKey::FractionantiB0Gen, "Fraction of generated B0bar" },
      { CrossSectionKey::FractionBplusGen, "Fraction of generated B+" },
      { CrossSectionKey::FractionBminusGen, "Fraction of generated B-" },
      { CrossSectionKey::FractionBs0Gen, "Fraction of generated Bs0" },
      { CrossSectionKey::FractionantiBs0Gen, "Fraction of generated anti-Bs0" },
      { CrossSectionKey::FractionbBarGen, "Fraction of generated b-Baryon" },
      { CrossSectionKey::FractionantibBarGen, "Fraction of generated anti-b-Baryon" },
      { CrossSectionKey::FractionBcplusGen, "Fraction of generated Bc+" },
      { CrossSectionKey::FractionBcminusGen, "Fraction of generated Bc-" },
      { CrossSectionKey::FractionB0Acc, "Fraction of accepted B0" },
      { CrossSectionKey::FractionantiB0Acc, "Fraction of accepted B0bar" },
      { CrossSectionKey::FractionBplusAcc, "Fraction of accepted B+" },
      { CrossSectionKey::FractionBminusAcc, "Fraction of accepted B-" },
      { CrossSectionKey::FractionBs0Acc, "Fraction of accepted Bs0" },
      { CrossSectionKey::FractionantiBs0Acc, "Fraction of accepted anti-Bs0" },
      { CrossSectionKey::FractionbBaryonAcc, "Fraction of accepted b-Baryon" },
      { CrossSectionKey::FractionantibBarAcc, "Fraction of accepted anti-b-Baryon" },
      { CrossSectionKey::FractionBcplusAcc, "Fraction of accepted Bc+" },
      { CrossSectionKey::FractionBcminusAcc, "Fraction of accepted Bc-" },

      { CrossSectionKey::FractionD0Gen, "Fraction of generated D0" },
      { CrossSectionKey::FractionantiD0Gen, "Fraction of generated D0bar" },
      { CrossSectionKey::FractionDplusGen, "Fraction of generated D+" },
      { CrossSectionKey::FractionDminusGen, "Fraction of generated D-" },
      { CrossSectionKey::FractionDsplusGen, "Fraction of generated Ds+" },
      { CrossSectionKey::FractionDsminusGen, "Fraction of generated Ds-" },
      { CrossSectionKey::FractioncBarGen, "Fraction of generated c-Baryon" },
      { CrossSectionKey::FractionanticBarGen, "Fraction of generated anti-c-Baryon" },
      { CrossSectionKey::FractionD0Acc, "Fraction of accepted D0" },
      { CrossSectionKey::FractionantiD0Acc, "Fraction of accepted D0bar" },
      { CrossSectionKey::FractionDplusAcc, "Fraction of accepted D+" },
      { CrossSectionKey::FractionDminusAcc, "Fraction of accepted D-" },
      { CrossSectionKey::FractionDsplusAcc, "Fraction of accepted Ds+" },
      { CrossSectionKey::FractionDsminusAcc, "Fraction of accepted Ds-" },
      { CrossSectionKey::FractioncBaryonAcc, "Fraction of accepted c-Baryon" },
      { CrossSectionKey::FractionanticBarAcc, "Fraction of accepted anti-c-Baryon" },

      { CrossSectionKey::FractionBGen, "Fraction of generated B" },
      { CrossSectionKey::FractionBstarGen, "Fraction of generated B*" },
      { CrossSectionKey::FractionB2starGen, "Fraction of generated B**" },
      { CrossSectionKey::FractionBAcc, "Fraction of accepted B" },
      { CrossSectionKey::FractionBstarAcc, "Fraction of accepted B*" },
      { CrossSectionKey::FractionB2starAcc, "Fraction of accepted B**" },

      { CrossSectionKey::FractionDGen, "Fraction of generated D" },
      { CrossSectionKey::FractionDstarGen, "Fraction of generated D*" },
      { CrossSectionKey::FractionD2starGen, "Fraction of generated D**" },
      { CrossSectionKey::FractionDAcc, "Fraction of accepted D" },
      { CrossSectionKey::FractionDstarAcc, "Fraction of accepted D*" },
      { CrossSectionKey::FractionD2starAcc, "Fraction of accepted D**" },

      { CrossSectionKey::FractionSignal, "Fraction of generated signal events" },
      { CrossSectionKey::FractionantiSignal, "Fraction of generated anti-signal events" },
      { CrossSectionKey::PartCutEff, "Particle cut efficiency" },
      { CrossSectionKey::antiPartCutEff, "Anti-particle cut efficiency" } };

  const GaudiUtils::VectorMap<std::string, CrossSectionsFSR::CrossSectionKey> CrossSectionsFSR::s2csk = {
      { "MBCrossSection", CrossSectionKey::MBCrossSection },
      { "MeanNoZeroPUInt", CrossSectionKey::MeanNoZeroPUInt },
      { "MeanPUInt", CrossSectionKey::MeanPUInt },
      { "MeanPUIntAcc", CrossSectionKey::MeanPUIntAcc },
      { "FullEvtCut", CrossSectionKey::FullEvtCut },
      { "bCrossGen", CrossSectionKey::bCrossGen },
      { "ThreebCrossGen", CrossSectionKey::ThreebCrossGen },
      { "PromptBCrossGen", CrossSectionKey::PromptBCrossGen },
      { "cCrossGen", CrossSectionKey::cCrossGen },
      { "ThreecCrossGen", CrossSectionKey::ThreecCrossGen },
      { "PromptCCrossGen", CrossSectionKey::PromptCCrossGen },
      { "bAndcCrossGen", CrossSectionKey::bAndcCrossGen },
      { "bCrossAcc", CrossSectionKey::bCrossAcc },
      { "ThreebCrossAcc", CrossSectionKey::ThreebCrossAcc },
      { "PromptBCrossAcc", CrossSectionKey::PromptBCrossAcc },
      { "cCrossAcc", CrossSectionKey::cCrossAcc },
      { "ThreecCrossAcc", CrossSectionKey::ThreecCrossAcc },
      { "PromptCCrossAcc", CrossSectionKey::PromptCCrossAcc },
      { "bAndcCrossAcc", CrossSectionKey::bAndcCrossAcc },
      { "GenLevelCut", CrossSectionKey::GenLevelCut },
      { "FractionB0Gen", CrossSectionKey::FractionB0Gen },
      { "FractionantiB0Gen", CrossSectionKey::FractionantiB0Gen },
      { "FractionBplusGen", CrossSectionKey::FractionBplusGen },
      { "FractionBminusGen", CrossSectionKey::FractionBminusGen },
      { "FractionBs0Gen", CrossSectionKey::FractionBs0Gen },
      { "FractionantiBs0Gen", CrossSectionKey::FractionantiBs0Gen },
      { "FractionbBarGen", CrossSectionKey::FractionbBarGen },
      { "FractionantibBarGen", CrossSectionKey::FractionantibBarGen },
      { "FractionBcplusGen", CrossSectionKey::FractionBcplusGen },
      { "FractionBcminusGen", CrossSectionKey::FractionBcminusGen },
      { "FractionB0Acc", CrossSectionKey::FractionB0Acc },
      { "FractionantiB0Acc", CrossSectionKey::FractionantiB0Acc },
      { "FractionBplusAcc", CrossSectionKey::FractionBplusAcc },
      { "FractionBminusAcc", CrossSectionKey::FractionBminusAcc },
      { "FractionBs0Acc", CrossSectionKey::FractionBs0Acc },
      { "FractionantiBs0Acc", CrossSectionKey::FractionantiBs0Acc },
      { "FractionbBaryonAcc", CrossSectionKey::FractionbBaryonAcc },
      { "FractionantibBarAcc", CrossSectionKey::FractionantibBarAcc },
      { "FractionBcplusAcc", CrossSectionKey::FractionBcplusAcc },
      { "FractionBcminusAcc", CrossSectionKey::FractionBcminusAcc },
      { "FractionD0Gen", CrossSectionKey::FractionD0Gen },
      { "FractionantiD0Gen", CrossSectionKey::FractionantiD0Gen },
      { "FractionDplusGen", CrossSectionKey::FractionDplusGen },
      { "FractionDminusGen", CrossSectionKey::FractionDminusGen },
      { "FractionDsplusGen", CrossSectionKey::FractionDsplusGen },
      { "FractionDsminusGen", CrossSectionKey::FractionDsminusGen },
      { "FractioncBarGen", CrossSectionKey::FractioncBarGen },
      { "FractionanticBarGen", CrossSectionKey::FractionanticBarGen },
      { "FractionD0Acc", CrossSectionKey::FractionD0Acc },
      { "FractionantiD0Acc", CrossSectionKey::FractionantiD0Acc },
      { "FractionDplusAcc", CrossSectionKey::FractionDplusAcc },
      { "FractionDminusAcc", CrossSectionKey::FractionDminusAcc },
      { "FractionDsplusAcc", CrossSectionKey::FractionDsplusAcc },
      { "FractionDsminusAcc", CrossSectionKey::FractionDsminusAcc },
      { "FractioncBaryonAcc", CrossSectionKey::FractioncBaryonAcc },
      { "FractionanticBarAcc", CrossSectionKey::FractionanticBarAcc },
      { "FractionBGen", CrossSectionKey::FractionBGen },
      { "FractionBstarGen", CrossSectionKey::FractionBstarGen },
      { "FractionB2starGen", CrossSectionKey::FractionB2starGen },
      { "FractionBAcc", CrossSectionKey::FractionBAcc },
      { "FractionBstarAcc", CrossSectionKey::FractionBstarAcc },
      { "FractionB2starAcc", CrossSectionKey::FractionB2starAcc },
      { "FractionDGen", CrossSectionKey::FractionDGen },
      { "FractionDstarGen", CrossSectionKey::FractionDstarGen },
      { "FractionD2starGen", CrossSectionKey::FractionD2starGen },
      { "FractionDAcc", CrossSectionKey::FractionDAcc },
      { "FractionDstarAcc", CrossSectionKey::FractionDstarAcc },
      { "FractionD2starAcc", CrossSectionKey::FractionD2starAcc },
      { "FractionSignal", CrossSectionKey::FractionSignal },
      { "FractionantiSignal", CrossSectionKey::FractionantiSignal },
      { "PartCutEff", CrossSectionKey::PartCutEff },
      { "antiPartCutEff", CrossSectionKey::antiPartCutEff },
      { "Unknown", CrossSectionKey::Unknown } };

} // namespace LHCb
