from __future__ import print_function

import itertools

###############################################################################
# (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
from builtins import range

from Configurables import (
    AlgResourcePool,
    ConfigurableDummy,
    HiveDataBrokerSvc,
    HiveWhiteBoard,
    HLTControlFlowMgr,
)
from Gaudi.Configuration import *

evtslots = 2
evtMax = 4
threads = 2

Ts = [ConfigurableDummy("T{}".format(i)) for i in range(8)]
Fs = [ConfigurableDummy("F{}".format(i)) for i in range(8)]

for t in Ts:
    t.CFD = 1
for f in Fs:
    f.CFD = 0

whiteboard = HiveWhiteBoard("EventDataSvc", EventSlots=evtslots)
HLTControlFlowMgr().CompositeCFNodes = []

listofnodes = ["NONLAZY_AND", "NONLAZY_OR", "LAZY_AND", "LAZY_OR"]

helperindex = 0
for compNode in listofnodes:
    # once with a True node first
    HLTControlFlowMgr().CompositeCFNodes.append(
        (
            compNode + "_TF",
            compNode,
            [Ts[helperindex].name(), Fs[helperindex].name()],
            True,
        )
    )
    helperindex += 1
    # once with a False node first
    HLTControlFlowMgr().CompositeCFNodes.append(
        (
            compNode + "_FT",
            compNode,
            [Fs[helperindex].name(), Ts[helperindex].name()],
            True,
        )
    )
    helperindex += 1

HLTControlFlowMgr().CompositeCFNodes.insert(
    0,
    ("top", "NONLAZY_OR", [x[0] for x in HLTControlFlowMgr().CompositeCFNodes], False),
)

HLTControlFlowMgr().ThreadPoolSize = threads
HLTControlFlowMgr().OutputLevel = VERBOSE

HiveDataBrokerSvc().OutputLevel = DEBUG

print(HLTControlFlowMgr().CompositeCFNodes)

app = ApplicationMgr(
    EvtMax=evtMax,
    EvtSel="NONE",
    ExtSvc=[whiteboard],
    EventLoop=HLTControlFlowMgr(),
    TopAlg=Ts + Fs,
)

HiveDataBrokerSvc().DataProducers = app.TopAlg
