/*****************************************************************************\
* (c) Copyright 2000-2021 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#include "Kernel/PlatformInfo.h"

#include "GaudiKernel/System.h"

#include <algorithm>
#include <vector>

#ifndef BINARY_TAG
#  error BINARY_TAG macro not defined
#endif

namespace {
  // clang-format off
  // IMPORTANT: always add new platforms at the end of the list
  const std::vector<std::string> known_platforms = {
    "unknown"
  , "i686-slc5-gcc43-dbg"
  , "i686-slc5-gcc43-opt"
  , "x86_64-centos7-gcc49-dbg"
  , "x86_64-centos7-gcc49-do0"
  , "x86_64-centos7-gcc49-opt"
  , "x86_64-centos7-gcc62-dbg"
  , "x86_64-centos7-gcc62-do0"
  , "x86_64-centos7-gcc62-opt"
  , "x86_64-centos7-gcc62-opt+o3"
  , "x86_64-centos7-gcc7-opt"
  , "x86_64-centos7-gcc7-dbg"
  , "x86_64-slc5-gcc43-dbg"
  , "x86_64-slc5-gcc43-opt"
  , "x86_64-slc5-gcc46-dbg"
  , "x86_64-slc5-gcc46-opt"
  , "x86_64-slc5-icc11-dbg"
  , "x86_64-slc5-icc11-opt"
  , "x86_64-slc6-clang32-opt"
  , "x86_64-slc6-clang37-dbg"
  , "x86_64-slc6-clang37-opt"
  , "x86_64-slc6-clang38-opt"
  , "x86_64-slc6-clang39-dbg"
  , "x86_64-slc6-clang39-opt"
  , "x86_64-slc6-gcc46-dbg"
  , "x86_64-slc6-gcc46-opt"
  , "x86_64-slc6-gcc47-opt"
  , "x86_64-slc6-gcc48-dbg"
  , "x86_64-slc6-gcc48-do0"
  , "x86_64-slc6-gcc48-opt"
  , "x86_64-slc6-gcc49-dbg"
  , "x86_64-slc6-gcc49-do0"
  , "x86_64-slc6-gcc49-opt"
  , "x86_64-slc6-gcc61-opt"
  , "x86_64-slc6-gcc62-dbg"
  , "x86_64-slc6-gcc62-do0"
  , "x86_64-slc6-gcc62-opt"
  , "x86_64-slc6-icc13-opt"
  , "x86_64+avx-centos7-gcc62-dbg+ofast"
  , "x86_64+avx-centos7-gcc62-opt"
  , "x86_64+avx-centos7-gcc62-opt+o3"
  , "x86_64+avx-centos7-gcc62-opt+ofast"
  , "x86_64+avx-centos7-gcc62-opt+ofast+g"
  , "x86_64+avx+fma-centos7-gcc62-opt"
  , "x86_64+avx2-centos7-gcc62-opt"
  , "x86_64+avx2+fma-centos7-gcc62-opt"
  , "x86_64+avx2+fma-centos7-gcc62-opt+o3"
  , "x86_64-ubuntu1704-gcc63-opt"
  , "x86_64-centos7-clang50-opt"
  , "x86_64-centos7-clang50-dbg"
  , "x86_64+avx2+fma-centos7-gcc7-opt"
  , "x86_64-slc6-gcc7-opt"
  , "x86_64-slc6-gcc7-dbg"
  , "x86_64-centos7-gcc7-do0"
  , "x86_64-centos7-gcc7-dbg+o1+asan"
  , "x86_64-centos7-gcc7-dbg+o1+lsan"
  , "x86_64-centos7-gcc7-dbg+o1+ubsan"
  , "x86_64-centos7-gcc7-dbg+o1+tsan"
  , "x86_64-centos7-gcc8-dbg+o1+asan"
  , "x86_64-centos7-gcc8-dbg+o1+lsan"
  , "x86_64-centos7-gcc8-dbg+o1+ubsan"
  , "x86_64-centos7-gcc8-dbg+o1+tsan"
  , "x86_64-slc6-gcc8-opt"
  , "x86_64-slc6-gcc8-dbg"
  , "x86_64-slc6-gcc8-do0"
  , "x86_64-centos7-gcc8-opt"
  , "x86_64-centos7-gcc8-dbg"
  , "x86_64-centos7-gcc8-do0"
  , "x86_64+avx-centos7-gcc8-opt"
  , "x86_64+avx2+fma-centos7-gcc8-opt"
  , "x86_64-centos7-gcc7-dbg+asan"
  , "x86_64-centos7-gcc7-dbg+lsan"
  , "x86_64-centos7-gcc7-dbg+ubsan"
  , "x86_64-centos7-gcc7-dbg+tsan"
  , "x86_64-centos7-gcc8-dbg+asan"
  , "x86_64-centos7-gcc8-dbg+lsan"
  , "x86_64-centos7-gcc8-dbg+ubsan"
  , "x86_64-centos7-gcc8-dbg+tsan"
  , "x86_64-centos7-clang8-opt"
  , "x86_64-centos7-clang8-dbg"
  , "x86_64-centos7-gcc9-opt"
  , "x86_64-centos7-gcc9-dbg"
  , "x86_64-centos7-gcc9-do0"
  , "x86_64+avx-centos7-gcc9-opt"
  , "x86_64+avx2+fma-centos7-gcc9-opt"
  , "x86_64-centos7-gcc9-dbg+asan"
  , "x86_64-centos7-gcc9-dbg+lsan"
  , "x86_64-centos7-gcc9-dbg+ubsan"
  , "x86_64-centos7-gcc9-dbg+tsan"
  , "skylake_avx512+vecwid256-centos7-gcc9-opt"
  , "skylake_avx512+vecwid256-centos7-gcc9-opt+g"
  , "skylake_avx512+vecwid256-centos7-gcc9-dbg"
  , "skylake_avx512+vecwid256-centos7-gcc9-do0"
  , "x86_64-centos7-clang10-opt"
  , "x86_64-centos7-clang10-dbg"
  , "x86_64-centos7-gcc9+py3-opt"
  , "x86_64-centos7-gcc9+py3-dbg"
  , "x86_64-centos7-gcc10-opt"
  , "x86_64-centos7-gcc10-dbg"
  , "x86_64-centos7-gcc10-do0"
  , "x86_64+avx-centos7-gcc10-opt"
  , "x86_64+avx2+fma-centos7-gcc10-opt"
  , "x86_64-centos7-gcc10-dbg+asan"
  , "x86_64-centos7-gcc10-dbg+lsan"
  , "x86_64-centos7-gcc10-dbg+ubsan"
  , "x86_64-centos7-gcc10-dbg+tsan"
  , "skylake_avx512+vecwid256-centos7-gcc10-opt"
  , "skylake_avx512+vecwid256-centos7-gcc10-opt+g"
  , "skylake_avx512+vecwid256-centos7-gcc10-dbg"
  , "skylake_avx512+vecwid256-centos7-gcc10-do0"
  , "x86_64_v2-centos7-gcc9-opt"
  , "x86_64_v2-centos7-gcc9-dbg"
  , "x86_64_v2-centos7-gcc9-do0"
  , "x86_64_v3-centos7-gcc9-opt"
  , "x86_64_v4+vecwid256-centos7-gcc9-opt"
  , "x86_64_v2-centos7-clang11-opt"
  , "x86_64_v2-centos7-clang11-dbg"
  , "x86_64_v2-centos7-gcc10-opt"
  , "x86_64_v2-centos7-gcc10-dbg"
  , "x86_64_v2-centos7-gcc10-do0"
  , "x86_64_v3-centos7-gcc10-opt"
  , "x86_64_v4+vecwid256-centos7-gcc9-opt"
  , "x86_64_v2-centos7-gcc10-dbg+asan"
  , "x86_64_v2-centos7-gcc10-dbg+lsan"
  , "x86_64_v2-centos7-gcc10-dbg+ubsan"
  , "x86_64_v2-centos7-gcc10-dbg+tsan"
  , "x86_64_v3-centos7-gcc10fp-opt"
  , "x86_64_v4+vecwid256-centos7-gcc10-opt"
  , "x86_64_v2-centos7-clang12-opt"
  , "x86_64_v2-centos7-clang12-dbg"
  , "x86_64_v2-centos7-gcc11-opt"
  , "x86_64_v2-centos7-gcc11-dbg"
  , "x86_64_v2-centos7-gcc11-do0"
  , "x86_64_v3-centos7-gcc11-opt"
  , "x86_64_v2-centos7-gcc11-dbg+asan"
  , "x86_64_v2-centos7-gcc11-dbg+lsan"
  , "x86_64_v2-centos7-gcc11-dbg+ubsan"
  , "x86_64_v2-centos7-gcc11-dbg+tsan"
  , "x86_64_v3-centos7-gcc11fp-opt"
  , "x86_64_v4+vecwid256-centos7-gcc11-opt"
  , "x86_64_v3-centos7-gcc11-opt+g"
  , "x86_64_v2-centos7-gcc10+dd4hep-opt"
  , "x86_64_v2-centos7-gcc10+dd4hep-dbg"
  , "x86_64_v2-centos7-gcc11+dd4hep-opt"
  , "x86_64_v2-centos7-gcc11+dd4hep-dbg"
  , "x86_64_v3-centos7-gcc11+dd4hep-opt"
  , "x86_64_v2-centos7-clang12+dd4hep-opt"
  , "x86_64_v2-centos7-clang12+dd4hep-dbg"
  , "x86_64_v3-centos7-gcc11+dd4hep-dbg"
  , "x86_64_v3-centos7-gcc11+dd4hep-opt+g"
  , "x86_64_v2-centos7-gcc11+detdesc-opt"
  , "x86_64_v2-centos7-gcc11+detdesc-dbg"
  , "x86_64_v3-centos7-gcc11+detdesc-opt+g"
  , "x86_64_v2-centos9-gcc11-opt"
  , "x86_64_v2-centos9-gcc11-dbg"
  , "x86_64_v2-centos9-gcc12-opt"
  , "x86_64_v2-centos9-gcc12-dbg"
  , "x86_64_v2-centos9-clang12-opt"
  , "x86_64_v2-centos9-clang12-dbg"
  , "x86_64_v3-centos9-gcc11-opt"
  , "x86_64_v3-centos9-gcc11-dbg"
  , "x86_64_v3-centos9-gcc12-opt"
  , "x86_64_v3-centos9-gcc12-dbg"
  , "x86_64_v2-centos9-gcc11+detdesc-opt"
  , "x86_64_v2-centos9-gcc11+detdesc-dbg"
  , "x86_64_v2-centos9-gcc12+detdesc-opt"
  , "x86_64_v2-centos9-gcc12+detdesc-dbg"
  , "x86_64_v2-centos9-clang12-opt"
  , "x86_64_v2-centos9-clang12-dbg"
  , "x86_64_v3-centos9-gcc11+detdesc-opt+g"
  , "x86_64_v3-centos9-gcc11+detdesc-dbg"
  , "x86_64_v3-centos9-gcc12+detdesc-opt+g"
  , "x86_64_v3-centos9-gcc12+detdesc-dbg"
  , "x86_64_v2-centos7-clang12+detdesc-opt"
  , "x86_64_v3-centos9-gcc11+detdesc-opt"
  , "x86_64_v3-centos9-gcc11-opt+g"
  , "x86_64_v3-centos9-gcc12-opt+g"
  , "x86_64_v2-centos9-gcc11-do0"
  , "x86_64_v2-centos9-gcc12-do0"
  , "x86_64_v3-centos9-clang12-opt"
  , "x86_64_v3-centos9-clang12-dbg"
  , "armv8.1_a-centos7-gcc11-opt"
  , "armv8.1_a-centos7-gcc11-dbg"
  , "armv8.1_a-el9-gcc11-opt"
  , "armv8.1_a-el9-gcc11-dbg"
  , "armv8.1_a-el9-gcc12-opt"
  , "armv8.1_a-el9-gcc12-dbg"
  , "x86_64_v2-el9-clang12-dbg"
  , "x86_64_v2-el9-clang12-opt"
  , "x86_64_v2-el9-clang15-dbg"
  , "x86_64_v2-el9-clang15-opt"
  , "x86_64_v2-el9-gcc11-dbg"
  , "x86_64_v2-el9-gcc11-do0"
  , "x86_64_v2-el9-gcc11-opt"
  , "x86_64_v2-el9-gcc11+detdesc-dbg"
  , "x86_64_v2-el9-gcc11+detdesc-opt"
  , "x86_64_v2-el9-gcc12-dbg"
  , "x86_64_v2-el9-gcc12-do0"
  , "x86_64_v2-el9-gcc12-opt"
  , "x86_64_v2-el9-gcc12+detdesc-dbg"
  , "x86_64_v2-el9-gcc12+detdesc-opt"
  , "x86_64_v3-el9-clang12-dbg"
  , "x86_64_v3-el9-clang12-opt"
  , "x86_64_v3-el9-clang15-dbg"
  , "x86_64_v3-el9-clang15-opt"
  , "x86_64_v3-el9-gcc11-dbg"
  , "x86_64_v3-el9-gcc11-opt"
  , "x86_64_v3-el9-gcc11-opt+g"
  , "x86_64_v3-el9-gcc11+detdesc-dbg"
  , "x86_64_v3-el9-gcc11+detdesc-opt"
  , "x86_64_v3-el9-gcc11+detdesc-opt+g"
  , "x86_64_v3-el9-gcc12-dbg"
  , "x86_64_v3-el9-gcc12-opt"
  , "x86_64_v3-el9-gcc12-opt+g"
  , "x86_64_v3-el9-gcc12+detdesc-dbg"
  , "x86_64_v3-el9-gcc12+detdesc-opt"
  , "x86_64_v3-el9-gcc12+detdesc-opt+g"
  , "x86_64_v2-centos7-gcc12-opt"
  , "x86_64_v3-centos7-gcc12-opt+g"
  , "x86_64_v2-centos7-gcc12-dbg"
  , "x86_64_v2-centos7-gcc12+detdesc-opt"
  , "x86_64_v3-centos7-gcc12+detdesc-opt+g"
  , "x86_64_v2-centos7-gcc12+detdesc-dbg"
  , "x86_64_v3-el9-gcc12+cuda12_1-opt+g"
  , "x86_64_v2-centos7-gcc12-opt"
  // -- LCG_104 platforms --
  // , "x86_64_v2-centos7-gcc12-dbg"
  // , "x86_64_v3-centos7-gcc12-opt+g"
  , "x86_64_v3-centos7-gcc12-opt"
  , "x86_64_v4+vecwid256-centos7-gcc12-opt"
  , "x86_64_v2-centos7-gcc12-dbg+alubsan"
  , "x86_64_v2-centos7-gcc12-dbg+asan"
  , "x86_64_v2-centos7-gcc12-dbg+lsan"
  , "x86_64_v2-centos7-gcc12-dbg+tsan"
  , "x86_64_v2-centos7-gcc12-dbg+ubsan"
  // , "x86_64_v2-el9-gcc12-opt"
  // , "x86_64_v2-el9-gcc12-dbg"
  // , "x86_64_v2-el9-gcc12-do0"
  , "x86_64_v2-el9-gcc13-opt"
  , "x86_64_v2-el9-gcc13-dbg"
  , "x86_64_v2-el9-clang16-opt"
  , "x86_64_v2-el9-clang16-dbg"
  // , "x86_64_v3-el9-gcc12-opt"
  // , "x86_64_v3-el9-gcc12-opt+g"
  , "x86_64_v3-el9-gcc13-opt"
  , "x86_64_v3-el9-gcc13-opt+g"
  , "x86_64_v4+vecwid256-el9-gcc12-opt"
  , "x86_64_v4+vecwid256-el9-gcc13-opt"
  , "x86_64_v2-el9-gcc12-dbg+alubsan"
  , "x86_64_v2-el9-gcc12-dbg+asan"
  , "x86_64_v2-el9-gcc12-dbg+lsan"
  , "x86_64_v2-el9-gcc12-dbg+tsan"
  , "x86_64_v2-el9-gcc12-dbg+ubsan"
  , "x86_64_v2-el9-gcc13-dbg+alubsan"
  , "x86_64_v2-el9-gcc13-dbg+asan"
  , "x86_64_v2-el9-gcc13-dbg+lsan"
  , "x86_64_v2-el9-gcc13-dbg+tsan"
  , "x86_64_v2-el9-gcc13-dbg+ubsan"
  , "x86_64_v3-el9-gcc12+cuda12_1-opt"
  // , "x86_64_v3-el9-gcc12+cuda12_1-opt+g"
  , "x86_64_v3-el9-gcc12+cuda12_1-dbg"
  , "x86_64_v3-el9-gcc12+hip5-opt"
  , "x86_64_v3-el9-gcc12+hip5-dbg"
  , "x86_64_v3-el9-clang16+hip5-opt"
  , "x86_64_v3-el9-clang16+hip5-dbg"
  // , "armv8.1_a-el9-gcc12-opt"
  // , "armv8.1_a-el9-gcc12-dbg"
  , "armv8.1_a-el9-gcc13-opt"
  , "armv8.1_a-el9-gcc13-dbg"
  // , "x86_64_v2-centos7-gcc12+detdesc-opt"
  // , "x86_64_v2-centos7-gcc12+detdesc-dbg"
  , "x86_64_v3-centos7-gcc12+detdesc-opt"
  // , "x86_64_v3-centos7-gcc12+detdesc-opt+g"
  , "x86_64_v2-centos7-gcc12+detdesc-dbg+alubsan"
  // , "x86_64_v3-el9-gcc12+detdesc-opt+g"
  , "x86_64_v2-el9-gcc13+detdesc-opt"
  , "x86_64_v2-el9-gcc13+detdesc-dbg"
  , "x86_64_v3-el9-gcc13+detdesc-opt+g"
  , "x86_64_v2-el9-clang16+detdesc-opt"
  , "x86_64_v3-el9-gcc13+cuda12_4-opt+g"
  };
  // IMPORTANT: always add new platforms at the end of the list
  // clang-format on

  const std::string invalid_platform = "invalid";

  const std::vector<std::string> known_instruction_sets = {
      "80386", "SSE",     "SSE2",     "SSE3",    "Supplementary SSE3 (SSSE3)", "SSE4.1", "SSE4.2", "AVX",
      "AVX2",  "AVX512F", "AVX512VL", "AVX512BW" };
  const std::string unknown_instructions_set = "unknown";

// special trick to add quotes around a macro value
// see https://stackoverflow.com/a/20632065
#define STR_( X ) #X
#define STR( X ) STR_( X )
  const auto binary_tag_name = STR( BINARY_TAG );
#undef STR
#undef STR_

  const std::uint16_t binary_tag_id =
      ( find( begin( known_platforms ), end( known_platforms ), binary_tag_name ) == end( known_platforms ) )
          ? 0 // unknown
          : static_cast<std::uint16_t>( find( begin( known_platforms ), end( known_platforms ), binary_tag_name ) -
                                        begin( known_platforms ) );
} // namespace
namespace LHCb {
  std::uint16_t PlatformInfo::s_hostInstrSetLevel = static_cast<std::uint16_t>( System::instructionsetLevel() );

  PlatformInfo::PlatformInfo() : m_binaryId( binary_tag_id ), m_hostInstrSetLevel( s_hostInstrSetLevel ) {}

  PlatformInfo::PlatformInfo( std::uint16_t binaryId, std::uint16_t hostInstrSetLevel )
      : m_binaryId( binaryId ), m_hostInstrSetLevel( hostInstrSetLevel ) {}

  const std::string& PlatformInfo::binaryTag() const {
    return ( m_binaryId < known_platforms.size() ) ? known_platforms[m_binaryId] : invalid_platform;
  }

  const std::string& PlatformInfo::hostInstrSet() const {
    return ( m_hostInstrSetLevel < known_instruction_sets.size() ) ? known_instruction_sets[m_hostInstrSetLevel]
                                                                   : unknown_instructions_set;
  }
} // namespace LHCb
