/*****************************************************************************\
* (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MODULE utestMetaEnum
#include <boost/test/unit_test.hpp>

#include "Gaudi/Property.h"
#include "GaudiKernel/StatusCode.h"
#include "Kernel/meta_enum.h"
#include <iostream>
#include <sstream>

meta_enum_class( MyEnum, int, Unknown = 0, Case1, Case2, Case3, CaseLarge = 10000 )

    meta_enum_class_with_unknown( MyEnum2, int, Case2, Unknown = 0, Case1, Case2, Case3, CaseLarge = 10000 )

        BOOST_AUTO_TEST_CASE( test_meta_enum_printing ) {
  std::ostringstream output;
  output << MyEnum::Case1 << " " << MyEnum::CaseLarge;
  BOOST_CHECK( output.str() == "'Case1' 'CaseLarge'" );
  MyEnum value;
  auto   sc = parse( value, "Case2" );
  BOOST_CHECK( value == MyEnum::Case2 && sc == StatusCode::SUCCESS );
  sc = parse( value, "non existing" );
  BOOST_CHECK( value == MyEnum::Unknown && sc == StatusCode::FAILURE );
  MyEnum2 value2;
  sc = parse( value2, "non existing" );
  std::cout << value2 << "" << to_underlying( MyEnum2_meta.defaultValue ) << std::endl;
  BOOST_CHECK( value2 == MyEnum2::Case2 && sc == StatusCode::FAILURE );

  for ( MyEnum i : members_of<MyEnum>() ) std::cout << i << " = " << to_underlying( i ) << '\n';
}

BOOST_AUTO_TEST_CASE( test_meta_enum_property ) {
  // make sure we can define a property:
  Gaudi::Property<MyEnum2> prop{ "hello", MyEnum2::Case3 };
}

BOOST_AUTO_TEST_CASE( test_meta_enum_min_max ) {
  BOOST_CHECK_EQUAL( min_value_of<MyEnum>(), MyEnum::Unknown );
  BOOST_CHECK_EQUAL( max_value_of<MyEnum>(), MyEnum::CaseLarge );
}

BOOST_AUTO_TEST_CASE( test_meta_enum_to_underlying ) {
  BOOST_CHECK_EQUAL( to_underlying( MyEnum::Unknown ), 0 );
  BOOST_CHECK_EQUAL( to_underlying( MyEnum::Case1 ), 1 );
  BOOST_CHECK_EQUAL( to_underlying( MyEnum::Case3 ), 3 );
  BOOST_CHECK_EQUAL( to_underlying( MyEnum::CaseLarge ), 10000 );
}
