/*****************************************************************************\
* (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#pragma once

#include "Event/Track.h"

#include "GaudiKernel/IAlgTool.h"

#include <vector>

#include "DetDesc/IGeometryInfo.h"
namespace LHCb {
  class MCParticle;
  class MCHit;
  class State;
  class StateVector;
} // namespace LHCb

/**
 *  Interface class for IdealStateCreator (in Tr/TrackMCTools).
 *
 *  @author Eduardo Rodrigues (adaptations to new track event model)
 *  @date   2005-04-04
 *
 *  @author Rutger van der Eijk
 *  @date   4-7-2002
 */

struct IIdealStateCreator : extend_interfaces<IAlgTool> {

  /// Retrieve interface ID
  DeclareInterfaceID( IIdealStateCreator, 3, 0 );

  /// Create a state at a z-position from an MCParticle's entry/exit points
  virtual StatusCode createState( const LHCb::MCParticle* mcPart, double zrec, LHCb::State& pState,
                                  IGeometryInfo const& geometry ) const = 0;

  /// Create a state at a z-position from an MCHits entry/exit points
  virtual StatusCode createState( const LHCb::MCHit* mcHit, double zrec, LHCb::State& pState,
                                  IGeometryInfo const& geometry ) const = 0;

  /// Create a state at an MCParticle's vertex
  virtual StatusCode createStateVertex( const LHCb::MCParticle* mcPart, LHCb::State& pState ) const = 0;

  /// Create a statevector at a z-position from an MCParticle's entry/exit points
  virtual StatusCode createStateVector( const LHCb::MCParticle* mcPart, double zrec, LHCb::StateVector& pVec,
                                        IGeometryInfo const& geometry ) const = 0;

  /// Create a statevector at a z-position from an MCHits entry/exit points
  virtual StatusCode createStateVector( const LHCb::MCHit* mcHit, double zrec, LHCb::StateVector& pVec,
                                        IGeometryInfo const& geometry ) const = 0;

  /// Create a statevector at an MCParticle's vertex
  virtual StatusCode createStateVectorVertex( const LHCb::MCParticle* mcPart, LHCb::StateVector& pVec ) const = 0;

  /// returns vector of states for all related MCHits
  virtual StatusCode getMCHitStates( const LHCb::MCParticle& mcPart, std::vector<LHCb::State>& track ) const = 0;
};
