/*****************************************************************************\
* (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
// ============================================================================
// GaudiKernel
// ============================================================================
#include "GaudiKernel/MsgStream.h"
#include "GaudiKernel/StatusCode.h"
// ============================================================================
// local
// ============================================================================
#include "LoKi/ErrorReport.h"
#include "LoKi/Report.h"
// ============================================================================
/** @file
 *
 * Implementation file for class LoKi::Report
 *
 *  This file is a part of LoKi project -
 *    "C++ ToolKit  for Smart and Friendly Physics Analysis"
 *
 *  The package has been designed with the kind help from
 *  Galina PAKHLOVA and Sergey BARSUK.  Many bright ideas,
 *  contributions and advices from G.Raven, J.van Tilburg,
 *  A.Golutvin, P.Koppenburg have been used in the design.
 *
 *  @author Vanya BELYAEV ibelyaev@physics.syr.edu
 *  @date 2001-01-23
 */
// ============================================================================
/*  Print the error  message, return status code
 *  @param msg    error message
 *  @param st     status code
 *  @param mx     maximal number of printouts
 */
// ============================================================================
StatusCode LoKi::Report::Error( const std::string& msg, StatusCode st, const size_t mx ) {
  return LoKi::ErrorReport::instance().Error( msg, st, mx );
}
// ============================================================================
/*  Print the warning  message, return status code
 *  @param msg    warning message
 *  @param st     status code
 *  @param mx     maximal number of printouts
 */
// ============================================================================
StatusCode LoKi::Report::Warning( const std::string& msg, StatusCode st, const size_t mx ) {
  return LoKi::ErrorReport::instance().Warning( msg, st, mx );
}
// ============================================================================
/*  Print the message and return status code
 *  @param msg    warning message
 *  @param st     status code
 *  @param lev    print level
 *  @return       status code
 */
// ============================================================================
void LoKi::Report::Print( const std::string& msg, StatusCode st, const MSG::Level lev ) {
  LoKi::ErrorReport::instance().Print( msg, st, lev ).ignore();
}
// ============================================================================
/*  Create and (re)-throw the exception
 *  @see LoKi::Exception
 *  @exception LoKi::Exception always!
 *  @param msg    exception message
 *  @param exc    (previous) exception of type GaudiException
 *  @param sc     status code
 *  @return       status code (fictive)
 */
// ============================================================================
void LoKi::Report::Exception( const std::string& msg, const GaudiException& exc, StatusCode sc ) {
  LoKi::ErrorReport::instance().Exception( msg, exc, sc );
}
// ============================================================================
/*  Create and (re)-throw the exception
 *  @see GaudiException
 *  @exception GaudiException always!
 *  @param msg    exception message
 *  @param exc    (previous) exception of type std::exception
 *  @param sc     status code
 *  @return       status code (fictive)
 */
// ============================================================================
void LoKi::Report::Exception( const std::string& msg, const std::exception& exc, StatusCode sc ) {
  LoKi::ErrorReport::instance().Exception( msg, exc, sc );
}
// ============================================================================
/*  Create and throw the exception
 *  @see GaudiException
 *  @exception GaudiException always!
 *  @param msg    exception message
 *  @param sc     status code
 *  @return       status code (fictive)
 */
// ============================================================================
void LoKi::Report::Exception( const std::string& msg, StatusCode sc ) {
  LoKi::ErrorReport::instance().Exception( msg, sc );
}
// ============================================================================
//  make a report
// ============================================================================
void LoKi::Report::report() { LoKi::ErrorReport::instance().report().ignore(); }
// ============================================================================

// ============================================================================
// The END
// ============================================================================
