2017-10-25 LHCbSys v43r1
===
This version uses Gaudi v29r0 and LCG_91 with ROOT 6.10.06.

This version is released on `master` branch. The previous release on `master` branch  was `LHCb v43r0`. This release contains all the changes that were made in `LHCb v42r6` `LHCb v42r6p1` and `LHCb v42r6p2` (released on `2017-patches` branch, see corresponding release notes), as well as the additional changes described in this file.

## Change to compiler support
**As from this release, support for gcc7 is added**
Note that C++17 is not yet fully supported, waiting for support from ROOT

**[MR !836] Suppress legitimate -Wimplicit-fallthrough warnings from gcc7**

**[MR !830] Fix to allow compilation with gcc 7.1**

**[MR !804] Remove use of register keyword as storage classifier**


## New or changed functionality
**[MR !861] LHCbMath: backport some developments (and fixes) from ostap-standalone**
1. Fix for Lomont' next_float/next_double method. Other functors from this file were also rewritten properly.
2. Bernstein polynomials:
  * improve numerical stability of polinomial division
  * remove gcd  (better to keep it in python using proper efficient divmod from C++)
  * add functions ( splits, control polygon,  convex hulls, crossing  points...) that allow easy and efficient implementation of root-finding in python
3. B-spline
  * improve numerical stability
  * add knot insertion using Boehm's algorithm
  * add functions (split, knot inserions, contorl polygons,  convex hulls, crossing points) that allow  easy and  efficient implementation of root-finding in python
  * add spline VDM approximatinon
  * add spline interpolation
  * add Berstein -> BSpline conversion
  * make some internal algorithm "visible" and usable outside b-spline ( de Boor, Boehm, ...)
4. Add few useful functions&objects into LHCb::Math namespace

**[MR !848] LHCbMath - Add AVX Guard class**
Simple AVX 'guard' class that calls `_mm256_zeroupper()` when it goes out of scope

**[MR !793] Update default DDDB tags according to LHCBCNDB-646**
Introduces new `ParticleTable.txt` in DDDB with following global tags:
```
Name               Based on         Datatype
dddb-20170721      dddb-20160318    2010
dddb-20170721-1    dddb-20160318-1  2011
dddb-20170721-2    dddb-20150928    2012,2013
dddb-20170721-3    dddb-20150724    2015,2016,2017
```

**[MR !777] New linker base class in Associators/AssociatorsBase**
See LHCBPS-1742

**[MR !905] Restore the default behaviour of AlgContextSvc**
The behaviour was changed in Gaudi v29r0, see gaudi/Gaudi!409. This change is needed in order to be able to determine the parent of public tools during intitialize (mostly for LoKi functors).

## Code optimisations
**[MR !856] Update VectorClass**
1. Update `VectorClass` to latest version.
2. Add the missing `instrset_detect.cpp` file to the library.
3. Explicitly link `LHCbMath` against `VectorClassLib` to pick up `instrset_detect()` from there.
4. Update `CPUDispatch.h` with `AVX512` enums.

**[MR !713] Disable Boost pool allocators**

**[MR !806, !823] Improve error detection & handling in TCK I/O**

**[MR !926] Optimize Solid{Cons,Polycone,SolidTubs}**
- avoid repeated sin/cos calls
- hint the compiler to use sincos by computing sin and cos right next to each other

## Bug fixes, protections and cleanups

**[MR !863] LoKiMC&LoKiGen: remove (hide) possibility to access Child selector by simple indices**
See LHCBPS-1745

**[MR !831] Fix Coverity warning: remove unnecessary include**

**[MR !842] Properly initialize MuonODEData in c'tor**

**[MR !825] L0DUReportMonitor: Fix repetitive label reset in monitoring**
Fix a missing reset that cause the histo label to be reset each event (for online monitoring purpose only)

**[MR !824] Fix git create database**

**[MR !822] Fix test_conddbui.py**

**[MR !815] Avoid bad query to PropertyConfigSvc**

**[MR !796] Restore short-circuiting for And and Or in LoKi/Primitives.h**
Fix for LHCBPS-1743

**[MR !789] Augment dictionary to fix stripping cache generation**

**[MR !206] Check whether args were passed to AuxFunBase prior to using them in toCpp**

**[MR !884] Use everywhere (consistently) LHCbMath/Lomont.h**
Removed from Gaudi with gaudi/Gaudi!398

**[MR !888] Make sure libgit2 can be found even if LCG does not provide openssl**

**[MR !899] Remove Kernel/TemplatedGenVector**
Now available from ROOT 6.10.x

**[MR !909] Fix untested StatusCodes uncovered by gaudi/Gaudi!386**

**[MR !911] Truncate existing raw files when writing**
Currently, if a raw file exists, it is not truncated when opened for writing. If the existing file has a larger size, the leftover bytes at the end will be left there and further reading will lead to segvs due to the corrupted file.
This MR simply adds the O_TRUNC flag to the open call and a test that verifies the correct behaviour.

**[MR !916] LoKiCore fix for regression discussed in Phys!206**

## Monitoring changes
**[MR !778] Update L0DU Bank monitor in case of Condition number exceeds 32**
Update L0DURawBankMonitor to handle not-reported conditions (when number of L0 conditions exceeds 32). Only affects the online monitoring in Orwell.


## Code modernisation
**[MR !866] Modernize lomont**
- Make constructors constexpr (again)
- Do not explicitly inhibit default constructor

**[MR !858] Modernize TrackEvent**
- remove `virtual` specification from LHCb::Track member functions
- remove use of deprecated (and removed in C++17) std::{unary,binary}_function
- remove unneccessary include of stdio.h and functional
- change functors to be non-template classes (structs) with a templated operator() -- this avoids having to specify a type when instantiating an instance of a functor
- remove commented out old code
- avoid unneccessary statics
- simplify Track::addToStates implementation -- no need to reserve first, do not modify the argument
- optimize Track::copy -- skip resetting data to zero when it will subsequently be overwritten

**[MR !851] Use gsl::span instead of bare pointers to non-owned arrays in Similarity**

**[MR !826] Modernize hltservices**
- replace CloseListener with a generic listener implementation
- make operator bool explicit
- rename flush in close
- do not use using namespace std::
- prefer pass-by-value followed by move

**[MR !847] Modernize MuonDAQ**
- remove `private` methods from IMuonRawBuffer interface
- add some `const` in IMuonRawBuffer interface
- prefer std::array over C-style arrays
- prefer zero initialization of array over .fill(0)
- make operator<< friend, in class scope, instead of in global scope
- make static constants constexpr
- prefer inherited constructors
- prefer switch/case over if-then
- prefer range-based loops
- prefer Gaudi::Property with direct member initialization over declareProperty
- reduce scope of variables as much as possible: only declare variables at the point they can be assigned a value
- use lambda to consolidate repeated code
- simplify a few arithmetic expressions
- prefer STL algorithms over raw loops
- use a scoped guard over explicitly grouped 'set/reset' calls
- prefer std::bitset over arrays of bool

**[MR !852] Clarify ownership transfer in CaloHypo::setPosition**

**[MR !821] Modernize LoKi Primitives**
Eliminate the need for Primitives.h to include itself after some pre-processor manipulations. This is done by avoiding the dedicated specialization for the 'void' functors (i.e. the case where there is no argument). Instead, use variadic number of arguments, and make both the case with and without argument a specialization.

In addition, implement `Min` and `Max` in terms of the generic `BinaryOp` -- which implies moving the handling of 3 or more functions into the `min` and `max` helpers in Math.h -- and `Round` in terms of the generic `UnaryOp`

**[MR !816] Modernize TrackParameters, TrackUse**
- use constexpr if possible
- remove deprecated std::unary_function
- prefer direct member initialization
- prefer STL algortihms over raw loops

**[MR !794] Modernize HltInterfaces**
- avoid mutable 'digest' data members
- prefer enum class for kinds (IAlgorithm,IService,IAlgTool) instead of strings
- make binary operators friend functions
- prefer STL algorithms over raw loops
- prefer boost::string_ref over const std::string& to avoid string copies

**[MR !798, !784] Modernize LoKiCore**
- introduce LOKI_REQUIRES macro to simplify SFINAE constraints
- prefer typename in template declarations
- introduce generic BindBinary functor, and use it to implement
    - EqualToValue
    - NotEqualToValue
    - LessThanValue
    - LessOrEqualValue
    - GreaterThanValue
    - GreaterOrEqualValue
    - MultiplyByValue
    - SumByValue
    - Minus1
    - Minus2
    - Divide1
    - Divide2
- remove fake argument in Round::Round
- add final to help the compiler de-virtualize
- remove SVN/CVS identifiers
- simplify LoKi::AddRef
- simplify LoKi::Const
- consolidate AlgFunctors: use baseclass + template implementation with a small struct to specialize
- add variadic min, max functors
- constrain FunctorArray constructor so it does not classify as copy c'tor
- generalize BinaryOp, and use it instead of Combination
- remove deprecated LoKi::select
- use Synced<T> to guarantee that non-const static data is never accessed without lock

**[MR !795, !792] Simplify DAQKernel**
- prefer inherited constructors
- put all related explicit template instantiations in a single file
- eliminate the need for getIfExistRaw
- remove obsolete SVN/CVS identifiers
- prefer =default over explicit implementations

**[MR !785] Add tagged_bool and use it to make ITrajPoca::minimize more explicit**

**[MR !782] Modernize bankKiller, OdinTypesFilter, OdinBCIDFilter**
Use Gaudi::Property, and use the opportunity to replace string options by dedicated types

**[MR !787] Modernize LHCbKernel**
- prefer std::array over boost::array
- prefer auto
- prefer std::greater<> over std::greater<T>

**[MR !672] Modernize L0DUBase: replace C-style arrays by std::array**

**[MR !912] Improve type punning**
- Introduce pun_to for type punning
- pun_to avoid the need to rely on C99 compatibility (accessing the 'non-active' member of a union is undefined behaviour in C++ -- but g++ behaves according to C99 in this respect)
- also, remove unneeded #include
- also, fully qualify enum values

## Changes to tests

**[MR !797] Add test for JIRA-LHCBPS-1743**

**[MR !908] testvdtmath_AVX2FMA.qmt: Ignore tiny difference in vdt::fast_cossin when FMA enabled**
