/*****************************************************************************\
* (c) Copyright 2000-2018 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#include "Event/UTTELL1BoardErrorBank.h"
#include "GaudiKernel/IRegistry.h"
#include <iostream>
#include <numeric>
#include <string>

//-----------------------------------------------------------------------------
// Implementation file for class : Detector::UT::ChannelID
//
// 2008-06-07 : M Needham
//-----------------------------------------------------------------------------

double LHCb::UTTELL1BoardErrorBank::fractionOK( const unsigned int pcn ) const {

  auto frac = std::accumulate( m_errorInfo.begin(), m_errorInfo.end(), 0.0,
                               [&]( double f, const auto* eI ) { return f + eI->fractionOK( pcn ); } );
  return frac / m_errorInfo.size();
}

std::string LHCb::UTTELL1BoardErrorBank::regName() const {
  // get the name of the container in the TES
  return parent()->registry()->identifier();
}

bool LHCb::UTTELL1BoardErrorBank::isUT() const { return regName().find( "UT" ) != std::string::npos; }

std::ostream& LHCb::UTTELL1BoardErrorBank::fillStream( std::ostream& s ) const {
  s << "Error bank for TELL1 board source " << key() << '\n';

  unsigned int pp = 0;
  for ( const auto& eI : m_errorInfo ) { s << "pp " << pp++ << '\n' << *eI << '\n'; }

  return s << std::endl;
}
